/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: basicfuncs.c,v 1.4 2007/01/04 20:53:57 piru Exp $
 
    Desc: Functions for readin .font files
    Lang: English.
*/

/****************************************************************************************/

#include <dos/dos.h>
#include <utility/tagitem.h>
#include <proto/exec.h>
#include <proto/utility.h>

#include <aros/debug.h>

#include "diskfont_intern.h"

/****************************************************************************************/

APTR AllocSegment(APTR *prevsegment_ptr, ULONG segmentsize, ULONG memflags,
                  struct DiskfontBase_intern *DiskfontBase)
{
	UBYTE *mem;

	segmentsize += sizeof(ULONG) + sizeof(BPTR);

	if ((mem = AllocMem(segmentsize, memflags)))
	{
		APTR prevsegment;

		*((ULONG *)mem)++ = segmentsize;

		if ((prevsegment = *prevsegment_ptr))
		{
			((BPTR *)prevsegment)[-1] = MKBADDR(mem);
		}
		*((BPTR *)mem)++ = NULL;

		*prevsegment_ptr = (APTR) mem;
	}

	return (APTR)mem;
}

/****************************************************************************************/

#ifndef __MORPHOS__

VOID FreeSegList(BPTR seglist, struct DiskfontBase_intern *DiskfontBase)
{
	ULONG *seg;

	while (seglist)
	{
		seg = BADDR(seglist);

		seglist = *((BPTR *) seg);

		FreeMem(&seg[-1], seg[-1]);
	}
}

#endif

/****************************************************************************************/

struct TagItem *ReadTags(BPTR fh, ULONG numtags, struct DiskfontBase_intern *DiskfontBase)
{
	struct TagItem  *taglist,
				*tagptr;

	D(bug("ReadTags(fh=%p, numtags=%d)\n", fh, numtags));

	/* Allocate memory for the tags */

	taglist = AllocateTagItems(numtags);

	if (!taglist)
		goto rt_failure;

	tagptr = taglist;

	/* Read the taglist into the buffer */

	for (; numtags --; )
	{
		ULONG val;

		if (!ReadLong( &DFB(DiskfontBase)->dsh, &val, (void *)fh))
			goto readfail;
		tagptr->ti_Tag = val;

		if (!ReadLong( &DFB(DiskfontBase)->dsh, &val, (void *)fh ))
			goto readfail;
		tagptr->ti_Data = val;

		tagptr ++;
	}

	ReturnPtr ("ReadTags", struct TagItem*, taglist);

readfail:
	FreeTagItems(taglist);
rt_failure:
	ReturnPtr("ReadTags", struct TagItem*, FALSE);

}

/****************************************************************************************/

BOOL WriteTags(BPTR fh, struct TagItem *taglist, struct DiskfontBase_intern *DiskfontBase)
{

	struct TagItem *tag;

	D(bug("WriteTags(fh=%p, taglists=%p)\n", fh, taglist));

#ifdef AROSAMIGA
	for (; (tag = NextTagItem((struct TagItem **)&taglist)); )
#else
	for (; (tag = NextTagItem((const struct TagItem **)&taglist)); )
#endif
	{
		if (!WriteLong( &DFB(DiskfontBase)->dsh, tag->ti_Tag, (void *)fh ))
			goto wt_failure;

		if (!WriteLong( &DFB(DiskfontBase)->dsh, tag->ti_Data, (void *)fh))
			goto wt_failure;
	}
	WriteLong(&DFB(DiskfontBase)->dsh, TAG_DONE, (void *)fh);
	WriteLong(&DFB(DiskfontBase)->dsh, 0, (void *)fh);

	ReturnBool ("WriteTags", TRUE);

wt_failure:
	ReturnBool ("WriteTags", FALSE);
}

/****************************************************************************************/
